#!/bin/sh
##########################################################################################
# Program Name : FileOpera
# Version      : 1.00.A
# Started Date : August 28, 2004
# Description  :
# A front end for multiple Tcl scripts for operation on any text file.
#
##########################################################################################

#Information
array set program {
	name		"FileOpera"
	author		"Binny V Abraham"
	version		"1.00.A"
	email		"binnyva@hotmail.com"
	website		"http://www.geocites.com/binnyva"
}


############################ Global Variables ##########################
set file_path ""
set start_folder "C:/"
set home_folder [file dirname $argv0]

############################ Important Funtions ##########################
# Function : openFile
# Shows a file opening dialog and set the contents of the selected files in the
#		text area.
proc openFile { } {
	global file_path start_folder
	set types {
	{{All Files}        *         }
	{{Text Files} 		{.txt}	  }
	{{HTML Files}       {.html}   }
	{{HTML Files}       {.htm}    }
	{{JavaScript Files} {.js} 	  }
	{{C/C++ Files} 		{.cpp}	  }
	{{C/C++ Files} 		{.c}	  }
	{{C/C++ Files} 		{.h}	  }
	{{Perl Files} 		{.pl}	  }
	{{Tcl/Tk Files} 	{.tcl}	  }
	}
	set result [tk_getOpenFile -filetypes $types -initialdir $start_folder]
	if { $result != "" } {
		#Put the filename in the entry widget after clearing it
		.file_path delete 0 end
		.file_path insert end "$result"

		#But the contents in the contents region
		set in [open "$result" r]
		.contents delete 1.0 end
		.contents insert end [read $in]
		close $in

		.contents edit reset
		set file_path $result
	}
}

# Function : save
# Saves the changes to the current file.
proc save {} {
	global file_path
	set contents [.contents get 1.0 end]
	set out [open "$file_path" w]
	if { $out != 0 } {
		puts $out $contents
		flush $out
		stat "File saved to \"$file_path\""
	}
	close $out
}

# Function : saveAs
# Saves the changes to a new file.
proc saveAs {} {
	global start_folder

	set types {
	{{All Files}        *         }
	{{Text Files} 		{.txt}	  }
	{{HTML Files}       {.html}   }
	{{HTML Files}       {.htm}    }
	{{JavaScript Files} {.js} 	  }
	{{C/C++ Files} 		{.cpp}	  }
	{{C/C++ Files} 		{.c}	  }
	{{C/C++ Files} 		{.h}	  }
	{{Perl Files} 		{.pl}	  }
	{{Tcl/Tk Files} 	{.tcl}	  }
	}
	set new_path [tk_getSaveFile -filetypes $types -initialdir $start_folder]
	if { $new_path != "" } {
		set contents [.contents get 1.0 end]
		set out [open "$new_path" w]
		if { $out != 0 } {
			puts $out $contents
			flush $out
			stat "File saved to \"$new_path\""
		}
		close $out
	}
}

# Function : aboutBox
# Desplay a Message box with the credits
proc aboutBox { } {
	global program
	tk_messageBox -type ok -title "About FileOpera" -message \
"FileOpera V $program(version)

by Binny V Abraham

$program(email)
$program(website)"
}

##########################################################################################

# Function : makeButtons
# This will find which all operations are there in the system and make buttons accodingly
proc makeButtons { frm } {
	global home_folder

	set folder [file join $home_folder "Operations"]
	set files [lsort [glob -nocomplain [file join $folder *.ops] [file join $folder *.tcl]]]
	set id 1

	foreach fl $files {
		set file_name [file tail $fl]
		set file_name [file rootname $file_name]
		set opr_name $file_name

		#Open the file and get the details
		set error_flag [catch { set opr_in [open "$fl" r] } ]
		if { $error_flag == 0 } {
			#Initializations
			set function_flag 0
			set name_flag 0
			#Get the operation name and the name of the function
			while { $name_flag == 0 && $function_flag == 0 } {
				set line [gets $opr_in]
				if { $line == -1 } {
					break ;# Get out at end of file
				}
				#Get the operation name
				if {!$name_flag} {
					set name_flag [regexp "#Name : (.*)" $line temp opr_name]
				}
				#Get the name of the function
				if {!$function_flag} {
					set function_flag [regexp "proc (.*) \{.*\} \{" $line temp opr_proc]
				}
			}
		}
		close $opr_in

		if { $function_flag } {
			source [file join "$fl"]
		
			set but [button $frm.but_opr$id -text "$opr_name" -command "$opr_proc"]
			grid $but -in $frm -row 1 -column $id -padx 15

			incr id ;# New ID for next file
		}
	}
}

#Show the specified text as the status for a second and then clear it
proc stat { msg } {
	.statusbar configure -text "$msg"
	after 2000 { .statusbar configure -text "Ready" }
}
#Update the status bar
proc status { msg } {
	if { $msg != "" } {
		.statusbar configure -text "$msg"
	} else {
		.statusbar configure -text "Ready"
	}
}

################################# Operation Functions ##############################
# These functions are used by the Operation files.
proc getFile {} {
	global file_path
	#set file_path [.file_path get]
	#Check wether it is a valid file
	if { [file exists $file_path] } {
		return $file_path
	} else {
		return ""
	}
}

# This will reset the variable selectors back to their original forms
proc defaulter {} {
	global var3 var4
	set var3 0
	set var4 0
	set c ".config"
	$c.one.lab_1 configure -text "Variable 1" -state disabled
	$c.one.ent_1 delete 0 end
	$c.one.ent_1 configure -state disabled
	$c.two.lab_2 configure -text "Variable 2" -state disabled
	$c.two.ent_2 delete 0 end
	$c.two.ent_2 configure -state disabled
	$c.chk_3 configure -text "Variable 3" -state disabled
	$c.scl_4 configure -label "Variable 4" -state disabled
	$c.but_apply configure -command "" -state disabled
}

#################################### GUI Building ##################################

#ToolBar to the Left
frame .top
frame .toolbar
button .toolbar.open_file -text "Open" -command { openFile }
button .toolbar.save_file -text "Save" -command { save }
button .toolbar.save_as -text "Save As" -command { saveAs }
#button .toolbar.undo -text "Undo" -command { catch { .contents edit undo } }
button .toolbar.about -text "About" -command { aboutBox }
#button .toolbar.pref -text "Preferences" -command { peferences }

#File Contents
frame .frm_content
text .contents -undo true -exportselection true -wrap none \
	-yscrollcommand ".srly_content set" -xscrollcommand ".srlx_content set"
scrollbar .srly_content -command ".contents yview" -orient v
scrollbar .srlx_content -command ".contents xview" -orient h


frame .button_bar
makeButtons {.button_bar}

frame .config
frame .config.one
label .config.one.lab_1 -text "Variable 1" -state disable
entry .config.one.ent_1 -state disable
frame .config.two
label .config.two.lab_2 -text "Variable 2" -state disable
entry .config.two.ent_2 -state disable
checkbutton .config.chk_3 -text "Variable 3" -state disable -variable var3
scale .config.scl_4 -from 0 -to 100 -orient horiz -label "Variable 4" -variable var4 \
	-length 250 -state disable
button .config.but_apply -text "  Apply  " -state disable

#Status Bar
label .statusbar -text "Ready" -justify left
entry .file_path -width 50

#Geometry Management
#Toolbar 
grid .top -in . -row 1 -column 1 -sticky ew -columnspan 2
grid .toolbar -in .top -row 1 -column 1 -sticky n
grid .toolbar.open_file -in .toolbar -row 1 -column 1 -sticky w -pady 5 -padx 5
grid .toolbar.save_file -in .toolbar -row 2 -column 1 -sticky w -pady 5 -padx 5
grid .toolbar.save_as	-in .toolbar -row 3 -column 1 -sticky w -pady 5 -padx 5
grid .toolbar.about		-in .toolbar -row 4 -column 1 -sticky w -pady 15 -padx 5
#grid .toolbar.pref		-in .toolbar -row 4 -column 1 -sticky w -pady 5 -padx 5

#File Contents
grid .frm_content -in .top -row 1 -column 2
grid .contents -in .frm_content -row 1 -column 1 -sticky nsew
grid .srly_content -in .frm_content -row 1 -column 2 -sticky ns
grid .srlx_content -in .frm_content -row 2 -column 1 -sticky we

grid .button_bar -in . -row 2 -column 1 -columnspan 2 -pady 5 -sticky w

grid .config.one.lab_1 -in .config.one -row 1 -column 1
grid .config.one.ent_1 -in .config.one -row 2 -column 1
grid .config.two.lab_2 -in .config.two -row 1 -column 1
grid .config.two.ent_2 -in .config.two -row 2 -column 1
grid .config.one .config.chk_3 -in .config -row 1 -sticky w -padx 5
grid .config.two .config.scl_4 .config.but_apply -in .config -row 2 -padx 5
grid .config -in . -row 3 -column 1 -columnspan 2 -sticky w -pady 10
#Status Bar
grid .statusbar -in . -row 4 -column 1 -sticky w 
grid .file_path -in . -row 4 -column 2 -sticky e


#This happens at the beginning - not really neesary.
# proc init {} {
# 	global file_path
# 	set file_path "C:/Scripts/Under Constuction/Tcl/File Operations/Jokes.txt"
# 	.file_path insert end $file_path
# 	#But the contents in the contents region
# 	set in [open "$file_path" r]
# 	.contents insert end [read $in]
# 	close $in
# 	.contents edit reset
# }
# init

# Getting the file name to open from the command line.
if { [lindex $argv 0] != "" } {
	set file_path [lindex $argv 0]
	.file_path insert end $file_path

	#But the contents in the contents region
	set in [open "$file_path" r]
	if { $in != 0 } { .contents insert end [read $in] }
	close $in
}

wm title . "FileOpera"
bind . <Key-Escape> {exit}
focus .